/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <unistd.h>
#include "disk-burn-window.h"
#include "genrecif.h"



enum {
    PROP_0,
    PROP_UNIX_DEVICE,
    PROP_FILE_PATH
};

typedef enum {
    ISO9660  = (1<<0),
    UDF_CD_OR_DVD = (1<<1),
    UDF_USB_DRIVE = (1<<2)
} FlieSystemMode;

typedef struct
{
    GtkWidget *name_label;
    GtkWidget *item_entry;
    GtkWidget *system_label;
    GtkWidget *system_combo;
    GtkWidget *speed_label;
    GtkWidget *speed_combo;
    GtkWidget *check_button;
    GtkWidget *progress_bar;
    GtkWidget *burn_button;
    char *unix_device;
    char *file_path;
} DiskBurnWindowPrivate;

struct _DiskBurnWindow
{
    GtkWindow parent;

    DiskBurnWindowPrivate *priv;
};

static char *tmpdevicepath = NULL;
static char *tmpdirnamepath = NULL;

G_DEFINE_TYPE_WITH_PRIVATE (DiskBurnWindow, disk_burn_window, GTK_TYPE_WINDOW)


static gboolean
css_provider_load_from_resource (GtkCssProvider *provider,
                     const char     *resource_path,
                     GError        **error)
{
    GBytes  *data;
    gboolean retval;

    data = g_resources_lookup_data (resource_path, 0, error);
    g_print ("%s resource_path: %s, data: %p\n", __func__, resource_path, data);
    if (!data) {
        if (!data) {
            return FALSE;
        }
    }

    retval = gtk_css_provider_load_from_data (provider,
                            g_bytes_get_data (data, NULL),
                            g_bytes_get_size (data),
                            error);
    g_bytes_unref (data);

    return retval;
}

static void
add_css_provider_at_priority (const gchar *rpath, guint priority)
{
    g_print ("\n%s %s\n", __func__, rpath);
    GtkCssProvider *provider;
    GError *error = NULL;

    provider = gtk_css_provider_new ();
    if (!css_provider_load_from_resource (provider, rpath, &error)) {
            g_warning ("Failed to load fallback css file: %s", error->message);
            if (error->message != NULL)
                g_error_free (error);
            goto out;
        }

    gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
                                                GTK_STYLE_PROVIDER (provider),
                                                priority);

out:
    g_object_unref (provider);
}

static void
init_icons_and_styles (void)
{
    gchar *theme_name;
    /* initialize search path for custom icons */
    gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
                       BURN_DATADIR G_DIR_SEPARATOR_S "icons");
    g_print ("%s %s\n", __func__, BURN_DATADIR G_DIR_SEPARATOR_S "icons");

    add_css_provider_at_priority ("/org/burn/disk-burn-window.css",
                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);


    g_object_get (gtk_settings_get_default (),
                  "gtk-theme-name", &theme_name,
                  NULL);
    g_print ("%s %s\n", __func__, theme_name);

    g_free (theme_name);
}

/* 
static void
entry_activate_callback (GtkWidget *entry, gpointer  user_data)
{
    const char *entry_text;

    entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
    if (entry_text != NULL && *entry_text != '\0') {
        g_print ("%s %s \n", __func__, entry_text);
    }
}
 */
static void
file_system_combo_changed (GtkWidget *combo, GtkWidget *widget)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    char *name;
    FlieSystemMode mode;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));

    if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo),  &iter)) {
        return;
    }
    gtk_tree_model_get (model, &iter, 0, &name, 1, &mode, -1);
    g_print ("%s %s %d\n", __func__, name, mode);

}

static GtkWidget*
create_file_system_combo (void)
{
    g_print ("%s\n", __func__);
    GtkWidget *item_combo;
    GtkListStore *store;
    GtkCellRenderer *cell;
    GtkTreeIter iter;

    store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
    item_combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
    g_object_unref (store);

    cell = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (item_combo), cell, TRUE);
    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (item_combo), cell,
                                    "text", 0,
                                    NULL);

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("ISO9660(start rock ridge and joliet)"), 1, ISO9660, -1);
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("UDF(compatibility mode of Windows CD/DVD)"), 1, UDF_CD_OR_DVD, -1);
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("UDF(compatibility mode of Windows USB Drive)"), 1, UDF_USB_DRIVE, -1);

    gtk_combo_box_set_active (GTK_COMBO_BOX (item_combo), 0);

    g_signal_connect (item_combo, "changed",
                    G_CALLBACK(file_system_combo_changed),
                    NULL);

    gtk_widget_show (item_combo);

    return item_combo;
}

static void
write_speed_combo_changed (GtkWidget *combo, GtkWidget *widget)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    char *name;
    int speed;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));

    if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo),  &iter)) {
        return;
    }
    gtk_tree_model_get (model, &iter, 0, &name, 1, &speed, -1);
    g_print ("%s %s %d\n", __func__, name, speed);

}

static GtkWidget*
create_write_speed_combo (void)
{
    g_print ("%s\n", __func__);
    GtkWidget *item_combo;
    GtkListStore *store;
    GtkCellRenderer *cell;
    GtkTreeIter iter;

    store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
    item_combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
    g_object_unref (store);

    cell = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (item_combo), cell, TRUE);
    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (item_combo), cell,
                                    "text", 0,
                                    NULL);

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("MAX"), 1, 12, -1);
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("8x"), 1, 8, -1);
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("4x"), 1, 4, -1);
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("2x"), 1, 2, -1);
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, _("1x"), 1, 1, -1);

    gtk_combo_box_set_active (GTK_COMBO_BOX (item_combo), 0);

    g_signal_connect (item_combo, "changed",
                    G_CALLBACK(write_speed_combo_changed),
                    NULL);

    gtk_widget_show (item_combo);

    return item_combo;
}

static void
data_check_clicked_callback (GtkWidget *button, gpointer user_data)
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))){
        g_print ("%s TRUE\n", __func__);
    } else {
        g_print ("%s FALSE\n", __func__);
    }
}

static void
cancel_button_clicked_callback (GtkWidget *button, DiskBurnWindow *window)
{
    g_print ("%s clicked\n", __func__);

    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (window->priv->progress_bar), 0);
    gtk_widget_set_sensitive (window->priv->name_label, TRUE);
    gtk_widget_set_sensitive (window->priv->system_label, TRUE);
    gtk_widget_set_sensitive (window->priv->speed_label, TRUE);
    gtk_widget_set_sensitive (window->priv->item_entry, TRUE);
    gtk_widget_set_sensitive (window->priv->system_combo, TRUE);
    gtk_widget_set_sensitive (window->priv->speed_combo, TRUE);
    gtk_widget_set_sensitive (window->priv->check_button, TRUE);
    gtk_widget_set_sensitive (window->priv->burn_button, TRUE);
}
/*
static void run_image_burn_th()
{
    //制作镜像
    CopyOnlyDirFiles(tmpdirnamepath);
    MkImg4Burn(tmpdirnamepath);
    //开始刻录
    if (strcmp(GetBurnedDiscFS(),"") == 0)
        BurnImage2Disc(GettmpBurnImageName(),tmpdevicepath,0,0);
    else
        BurnImage2Disc(GettmpBurnImageName(),tmpdevicepath,1,0);
       //清理缓存文件
    ClearDiscBurnInfo();
    return ;
}
*/
//反复读取进度 直到刻录结束
static void run_set_burn_process_bar(DiskBurnWindow *window)
{
    int ret = 0;
    
    while (ret >= 0 && ret < 100 )
    {
        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (window->priv->progress_bar), ((double)ret)/((double)100));
        ret = GetBurnRate();
        sleep(1);
    }
    if (ret == 100)
    {
        printf("[run_set_burn_process_bar] burn end ret = 100 .\n");
        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (window->priv->progress_bar), 1);
    }
    else if (ret == -1)
    {//system执行失败 (系统资源不足)
        printf("[run_set_burn_process_bar] burn failed ret = -1 .\n");
    }
    else if (ret == -2)
    {// growisofs运行异常 (刻录进程异常终止)
        printf("[run_set_burn_process_bar] burn failed WIFEXITED(status) == 0.\n");
    }
    else if (ret == -3)
    {// growisofs运行异常 (没有安装改版的growisofs)
        printf("[run_set_burn_process_bar] burn failed, may be growisofs dont support -R .\n");	
    }
    else if (ret == -4)
    {// growisofs运行异常 (growisofs -R-Z用法有问题)
        printf("[run_set_burn_process_bar] burn failed, may be growisofs -R-Z wrong use .\n");	
    }
    //刻录完成最好可以根据结果弹窗提示是否成功
    /*
    GtkWidget  *dialog;
    dialog = gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,
    GTK_MESSAGE_INFO,GTK_BUTTONS_OK,"burn end");

	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
	gtk_widget_show_all (dialog);
    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
    */
    //刻录完成清理缓存文件
    
    ClearDiscBurnInfo();
    g_thread_exit(0);
    return;
}
//启动控制进度条显示线程
static void start_set_burn_process_bar(DiskBurnWindow *window)
{
    printf("[start_set_burn_process_bar] run_set_burn_process_bar start.\n");
	GThread *thread_run = g_thread_new ("burn start", run_set_burn_process_bar, window);

    return ;
}

static void
burn_button_clicked_callback (GtkWidget *button, DiskBurnWindow *window)
{
    g_print ("%s clicked\n", __func__);
    const char *entry_text;
    entry_text = gtk_entry_get_text (GTK_ENTRY (window->priv->item_entry));
    if (entry_text != NULL && *entry_text != '\0') {
        g_print ("%s %s \n", __func__, entry_text);
        SetDiscLabelName(entry_text);
    }

    GtkTreeIter iter;
    GtkTreeModel *model;
    char *system_fs, *speed_str;
    FlieSystemMode mode;
    int speed;
    gboolean data_check;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (window->priv->system_combo));
    if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (window->priv->system_combo),  &iter)) {
        return;
    }
    gtk_tree_model_get (model, &iter, 0, &system_fs, 1, &mode, -1);
    g_print ("%s system_fs:%s mode:%d\n", __func__, system_fs, mode);
    switch (mode) {
    case ISO9660:
        SetBurnFS4Disc("ISO9660");
        break;
    case UDF_CD_OR_DVD:
        SetBurnFS4Disc("UDF1.02");
        break;
    case UDF_USB_DRIVE:
        SetBurnFS4Disc("UDF2.01");
        break;
    default:
        SetBurnFS4Disc("UDF1.02");
        break;
    }

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (window->priv->speed_combo));
    if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (window->priv->speed_combo),  &iter)) {
        return;
    }
    gtk_tree_model_get (model, &iter, 0, &speed_str, 1, &speed, -1);
    g_print ("%s speed_str:%s speed:%d\n", __func__, speed_str, speed);
    SetBurnSpeed(speed);


    data_check = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(window->priv->check_button));
    g_print ("%s data_check: %s\n", __func__, data_check ? "TRUE" : "FALSE");
    SetVerifyAfterBurn(data_check);

    g_free (system_fs);
    g_free (speed_str);

    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (window->priv->progress_bar), 0);
    gtk_widget_set_sensitive (window->priv->name_label, FALSE);
    gtk_widget_set_sensitive (window->priv->system_label, FALSE);
    gtk_widget_set_sensitive (window->priv->speed_label, FALSE);
    gtk_widget_set_sensitive (window->priv->item_entry, FALSE);
    gtk_widget_set_sensitive (window->priv->system_combo, FALSE);
    gtk_widget_set_sensitive (window->priv->speed_combo, FALSE);
    gtk_widget_set_sensitive (window->priv->check_button, FALSE);
    gtk_widget_set_sensitive (window->priv->burn_button, FALSE);

    g_print ("%s unix_device: %s\n", __func__, window->priv->unix_device);

    g_print ("%s file_path: %s\n", __func__, window->priv->file_path);
    //制作镜像
    CopyOnlyDirFiles(window->priv->file_path);
    MkImg4Burn(window->priv->file_path);
    //开始刻录
    if (strcmp(GetBurnedDiscFS(),"") == 0)
        BurnImage2Disc(GettmpBurnImageName(),tmpdevicepath,0,0);
    else
        BurnImage2Disc(GettmpBurnImageName(),tmpdevicepath,1,0);
    start_set_burn_process_bar(window);

}


static void
disk_burn_window_init (DiskBurnWindow *window)
{
    g_print ("%s\n", __func__);
    DiskBurnWindowPrivate *priv;
    GtkWidget *header_bar;
    GtkWidget *icon_name_box, *label, *image;
    GtkWidget *vbox;
    GtkWidget *item_box;
    GtkWidget *cancel_button;
    int ret = 0;

    window->priv = priv = disk_burn_window_get_instance_private (window);

    // window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window, DIST_TYPE_BURN_WINDOW, DiskBurnWindowPrivate);

    init_icons_and_styles ();

    gtk_window_set_default_size (GTK_WINDOW (window), 492, 490);
    gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);

    header_bar = gtk_header_bar_new ();
    gtk_header_bar_set_decoration_layout (GTK_HEADER_BAR (header_bar), "menu:close");
    gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (header_bar), TRUE);
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (header_bar)), "burn-window-headerbar");
    gtk_window_set_titlebar (GTK_WINDOW (window), header_bar);
    gtk_widget_show (header_bar);


    image = gtk_image_new_from_resource ("/org/burn/icons/brasero.png");
    label = gtk_label_new (_("Burn"));
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (label)), "title-label");
    icon_name_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_set_spacing (GTK_BOX (icon_name_box), 12);
    gtk_box_pack_start (GTK_BOX (icon_name_box), image, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (icon_name_box), label, FALSE, FALSE, 0);
    gtk_header_bar_pack_start (GTK_HEADER_BAR (header_bar), icon_name_box);
    gtk_widget_show_all (icon_name_box);


    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 24);
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (vbox)), "burn-vbox");
    gtk_container_add (GTK_CONTAINER (window), vbox);
    // 初始化光盘
    ret = InitDisc(tmpdevicepath);
    if (ret == -1)
    {
        g_print ("[%s] InitDisc(%s) failed. \n", __func__,tmpdevicepath);
    }

    /* 光盘名称 */
    item_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
    gtk_box_pack_start (GTK_BOX (vbox), item_box, FALSE, FALSE, 0);

    priv->name_label = gtk_label_new (_("Disk Name:"));
    gtk_box_pack_start (GTK_BOX (item_box), priv->name_label, FALSE, FALSE, 0);
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (priv->name_label)), "item-label");

    priv->item_entry = gtk_entry_new ();
    //获取卷标
    char labelname[257]={'\0'};
    GetDiscLabelName(labelname);
    if (strlen(labelname) != 0)
        gtk_entry_set_text(priv->item_entry,labelname);
    
    // 输入框按下回车会触发这个函数
    // g_signal_connect (priv->item_entry, "activate", G_CALLBACK (entry_activate_callback), NULL);
    // gtk_box_pack_start (GTK_BOX (item_box), priv->item_entry, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (item_box), priv->item_entry, FALSE, FALSE, 0);

    /* 文件系统 */
    item_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
    gtk_box_pack_start (GTK_BOX (vbox), item_box, FALSE, FALSE, 0);

    priv->system_label = gtk_label_new (_("File System:"));
    gtk_box_pack_start (GTK_BOX (item_box), priv->system_label, FALSE, FALSE, 0);
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (priv->system_label)), "item-label");

    priv->system_combo = create_file_system_combo ();
    //获取光盘类型 如果是刻录过的盘才有
    char *discfs;
    discfs = GetBurnedDiscFS();
    if (strcmp(discfs,"ISO9660") == 0)
        gtk_combo_box_set_active (GTK_COMBO_BOX (priv->system_combo), 0);
    else if (strcmp(discfs,"UDF1.02") == 0)
        gtk_combo_box_set_active (GTK_COMBO_BOX (priv->system_combo), 1);
    else if (strcmp(discfs,"UDF2.01") == 0)
        gtk_combo_box_set_active (GTK_COMBO_BOX (priv->system_combo), 2);
    else if (strcmp(discfs,"") == 0)  //对于全新光盘建议使用udf1.02
        gtk_combo_box_set_active (GTK_COMBO_BOX (priv->system_combo), 1);

    // gtk_box_pack_start (GTK_BOX (item_box), priv->system_combo, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (item_box), priv->system_combo, FALSE, FALSE, 0);
    if (strcmp(discfs,"") != 0)
        gtk_widget_set_sensitive (window->priv->system_combo, FALSE);
    /* 写入速度 */
    item_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
    gtk_box_pack_start (GTK_BOX (vbox), item_box, FALSE, FALSE, 0);

    priv->speed_label = gtk_label_new (_("Write Speed:"));
    gtk_box_pack_start (GTK_BOX (item_box), priv->speed_label, FALSE, FALSE, 0);
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (priv->speed_label)), "item-label");

    priv->speed_combo = create_write_speed_combo ();
    // gtk_box_pack_start (GTK_BOX (item_box), priv->speed_combo, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (item_box), priv->speed_combo, FALSE, FALSE, 0);


    /* 校验数据 */
    priv->check_button = gtk_check_button_new_with_label(_("Data check"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->check_button), FALSE);
    g_signal_connect(priv->check_button, "clicked", G_CALLBACK(data_check_clicked_callback), NULL);

    gtk_box_pack_start (GTK_BOX (vbox), priv->check_button, FALSE, FALSE, 0);
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (priv->check_button)), "item-check-button");

    /* 进度条 */
    priv->progress_bar = gtk_progress_bar_new ();
    gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (priv->progress_bar), 0);
    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (priv->progress_bar), 0);
    gtk_box_pack_start(GTK_BOX (vbox), priv->progress_bar, FALSE, FALSE, 0);

    /* 按钮 */
    item_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
    gtk_box_pack_end (GTK_BOX (vbox), item_box, FALSE, FALSE, 0);

    priv->burn_button = gtk_button_new_with_label (_("Burn"));
    g_signal_connect (priv->burn_button, "clicked", G_CALLBACK(burn_button_clicked_callback), window);
    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (priv->burn_button)), "burn-button");
    gtk_box_pack_end (GTK_BOX (item_box), priv->burn_button, FALSE, FALSE, 0);

    cancel_button = gtk_button_new_with_label (_("Cancel"));
    g_signal_connect (cancel_button, "clicked", G_CALLBACK(cancel_button_clicked_callback), window);
    gtk_box_pack_end (GTK_BOX (item_box), cancel_button, FALSE, FALSE, 0);

    gtk_widget_show_all (GTK_WIDGET (window));
}


static void
disk_burn_window_get_property (GObject      *object,
                guint         prop_id,
                GValue *value,
                GParamSpec   *pspec)
{
    g_print ("%s\n", __func__);
    DiskBurnWindow *window;

    window = DISK_BURN_WINDOW (object);

    switch (prop_id) {

    case PROP_UNIX_DEVICE:
        g_value_set_string (value, window->priv->unix_device);
        break;
    case PROP_FILE_PATH:
        g_value_set_string (value, window->priv->file_path);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}

static void
disk_burn_window_set_property (GObject      *object,
                guint         prop_id,
                const GValue *value,
                GParamSpec   *pspec)
{
    g_print ("%s\n", __func__);
    DiskBurnWindow *window;

    window = DISK_BURN_WINDOW (object);

    switch (prop_id) {

    case PROP_UNIX_DEVICE:
        g_free (window->priv->unix_device);
        window->priv->unix_device = g_strdup (g_value_get_string (value));
        g_object_notify (object, "unix-device");
        g_print ("%s unix_device: %s\n", __func__, window->priv->unix_device);
        break;
    case PROP_FILE_PATH:
        g_free (window->priv->file_path);
        window->priv->file_path = g_strdup (g_value_get_string (value));
        g_object_notify (object, "file-path");
        g_print ("%s file_path: %s\n", __func__, window->priv->file_path);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
        break;
    }
}

static void
disk_burn_window_finalize (GObject *object)
{
    g_print ("%s\n", __func__);
    DiskBurnWindow *window;

    window = DISK_BURN_WINDOW (object);


    g_free (window->priv->unix_device);
    g_free (window->priv->file_path);

    G_OBJECT_CLASS (disk_burn_window_parent_class)->finalize (object);
}

static void
disk_burn_window_class_init (DiskBurnWindowClass *class)
{
    g_print ("%s\n", __func__);
    GObjectClass *object_class = G_OBJECT_CLASS (class);
    object_class->finalize = disk_burn_window_finalize;
    object_class->get_property = disk_burn_window_get_property;
    object_class->set_property = disk_burn_window_set_property;


    g_object_class_install_property (object_class,
                     PROP_UNIX_DEVICE,
                     g_param_spec_string ("unix-device",
                                  "Unix device",
                                  "Unix device",
                                  NULL,
                                  G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));

    g_object_class_install_property (object_class,
                     PROP_FILE_PATH,
                     g_param_spec_string ("file-path",
                                  "File path",
                                  "File path",
                                  NULL,
                                  G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
}

GtkWidget *
disk_burn_window_new (const char *unix_device, const char *file_path)
{

    g_print ("%s unix_device: %s file_path: %s\n", __func__, unix_device, file_path);
    if (tmpdevicepath == NULL)
        tmpdevicepath = malloc(strlen(unix_device) + 1);
    else
    {
        free(tmpdevicepath);
        tmpdevicepath = malloc(strlen(unix_device) + 1);
    }
    strcpy(tmpdevicepath,unix_device);
    if (tmpdirnamepath == NULL)
        tmpdirnamepath = malloc(strlen(file_path) + 1);
    else
    {
        free(tmpdirnamepath);
        tmpdirnamepath = malloc(strlen(file_path) + 1);
    }
    strcpy(tmpdirnamepath,file_path);

    return g_object_new (DIST_TYPE_BURN_WINDOW,
                         "unix_device", unix_device,
                         "file_path", file_path,
                         NULL);
}
